//
//  MapViewController.swift
//  Restaurants
//
//  Created by Radu Ionescu on 22/03/2017.
//  Copyright © 2017 lab. All rights reserved.
//

import UIKit
import CoreLocation
import MapKit

class Annotation: NSObject, MKAnnotation
{
    var coordinate: CLLocationCoordinate2D
    var title: String?
    var subtitle: String?
    var index: Int?
    
    init(coordinate: CLLocationCoordinate2D, title: String, subtitle: String)
    {
        self.coordinate = coordinate
        self.title = title
        self.subtitle = subtitle
    }
}

class MapViewController: UIViewController, MKMapViewDelegate
{
    var appModel: AppModel?
    
    @IBOutlet weak var mapView: MKMapView!
    @IBOutlet weak var activityIndicator: UIActivityIndicatorView!
    
    override func viewDidLoad()
    {
        super.viewDidLoad()
        self.appModel = AppModel.sharedModel
        self.mapView!.delegate = self
        
        self.activityIndicator.startAnimating()
        self.reloadMapAnnotations()
        
        NotificationCenter.default.addObserver(self,
                                               selector: #selector(MapViewController.reloadMapAnnotations),
                                               name: Notification.Name(rawValue:"didReceiveRestaurantsData"),
                                               object: appModel!)
    }
    
    override func didReceiveMemoryWarning()
    {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    func reloadMapAnnotations()
    {
        var meanCoordinates = (0.0, 0.0)
        mapView!.removeAnnotations(mapView!.annotations)
        
        for (i, obj) in (appModel!.restaurants?.enumerated())!
        {
            let restaurant = obj as! [String:Any]
            let title = restaurant["name"] as! String
            let subtitle = restaurant["category"] as! String
            let coordinate = CLLocationCoordinate2D(latitude: restaurant["latitude"] as!CLLocationDegrees,
                                                    longitude: restaurant["longitude"] as! CLLocationDegrees)
            let annotation = Annotation(coordinate:
                coordinate, title: title, subtitle: subtitle)
            annotation.index = i
            meanCoordinates.0 += coordinate.latitude
            meanCoordinates.1 += coordinate.longitude
            mapView!.addAnnotation(annotation)
        }
        
        meanCoordinates.0 /= Double((appModel!.restaurants?.count)!)
        meanCoordinates.1 /= Double((appModel!.restaurants?.count)!)
        let center = CLLocationCoordinate2D(latitude: meanCoordinates.0,
                                            longitude: meanCoordinates.1)
        
        let span = MKCoordinateSpan(latitudeDelta: 4.0 / 111.2,
                                    longitudeDelta: 4.0 / 111.2)
        
        let region = MKCoordinateRegion(center: center,
                                        span: span)
        mapView!.setRegion(region,
                           animated: true)
        
        if (appModel?.restaurants?.count ?? 0) > 0
        {
            self.activityIndicator!.stopAnimating()
        }
    }
    
    // MARK: - Map View Delegate Methods
    
    func mapView(_ mapView: MKMapView, viewFor annotation: MKAnnotation) -> MKAnnotationView?
    {
        var pinView = mapView.dequeueReusableAnnotationView(withIdentifier: "RestaurantPinView") as? MKPinAnnotationView
        
        if(pinView == nil)
        {
            pinView = MKPinAnnotationView(annotation: annotation, reuseIdentifier: "RestaurantPinView")
            pinView!.canShowCallout = true
        }
        
        pinView!.annotation = annotation
        pinView!.rightCalloutAccessoryView = UIButton(type:.detailDisclosure)
        
        return pinView
    }
    
    func mapView(_ mapView: MKMapView, annotationView view: MKAnnotationView, calloutAccessoryControlTapped control: UIControl)
    {
        let annotation = view.annotation as! Annotation
        let restaurant = appModel?.restaurants? [annotation.index!] as! [String:Any]
        
        let detailsViewController = self.storyboard?.instantiateViewController(withIdentifier: "DetailsTableViewController") as! DetailsTableViewController
        detailsViewController.restaurant = restaurant
        
        self.navigationController?.pushViewController(detailsViewController, animated: true)
    }
}
